
Rem
bbdoc: Klasse zum Herstellen einer Verbindung mit einem Server
End Rem
Type TNetClient
	Field _host:Byte Ptr
	Field _peer:Byte Ptr
	
	Rem
	bbdoc: Erstellt einen neuen, unverbundenen, Client
	End Rem
	Function Create:TNetClient()
		Local client:TNetClient = New TNetClient
		Return client
	End Function

	Rem
	bbdoc: Stellt eine Verbindung mit einem Server her
	returns: True wenn die Verbindung zu Stande kommt, False wenn nicht
	about:
	host:String			IP/Hostname des Servers
	port:Int			Port des Servers
	connectTimeout:Int	(Optional) Maximale Zeit in ms fr die Verbindungsherstellung.
	End Rem
	Method Connect:Int(host:String, port:Int, connectTimeout:Int = 5000)
		'In case we are connected -> disconnect
		Disconnect()
		
		'Create address struct
		Local addr:Byte Ptr = enet_address_create(0, port)
		If addr = Null Then Return False
		enet_address_set_host(addr, host)
		
		'Create host
		_host = enet_host_create(Null, 1, 0, 0)
		If _host = Null Then Return False

		'Connect to the remote host
		_peer = enet_host_connect(_host, addr, 2)
		enet_address_destroy(addr)

		'No peer available for connection?
		If _peer = Null Then
			enet_host_destroy(_host)
			_host = Null
			Return False
		EndIf
		
		Local ev:ENetEvent = New ENetEvent
		
		'Wait for connection response
		If enet_host_service(_host, ev, connectTimeout) > 0 And ev.event = ENET_EVENT_TYPE_CONNECT
			'Connection successful
			Return True
		Else
			'Timeout reached
			enet_peer_reset(_peer)
			enet_host_destroy(_host)
			_host = Null
			_peer = Null
			Return False
		End If
	End Method

	Rem
	bbdoc: Trennt die Verbindung mit dem Server
	about:
	disconnectTime:Int		(Optional) Maximale Zeit in ms, zum Senden allflliger letzter Nachrichten vor der Trennung
	End Rem
	Method Disconnect(disconnectTime:Int = 2000)
		'Already disconnected?
		If _host = Null And _peer = Null Then Return

		If _peer = Null Then
			enet_host_destroy(_host)
			_host = Null
			Return
		End If
		
		If _host = Null Then
			enet_peer_reset(_peer)
			_peer = Null
			Return
		End If
	
		'Immediate disconnect (no server notification)
		If disconnectTime <= 0 Then
			enet_peer_reset(_peer)
			enet_host_destroy(_host)
			_host = Null
			_peer = Null
			Return
		End If
		
		'Gentle disconnect (wait for server acknowledgement)
		Local ev:ENetEvent = New ENetEvent
		Local begin:Int = MilliSecs()

		enet_peer_disconnect(_peer)
		
		While enet_host_service(_host, ev, disconnectTime) > 0
			Select ev.event
				Case ENET_EVENT_TYPE_RECEIVE
					'Drop any received packets
					enet_packet_destroy(ev.packet)

				Case ENET_EVENT_TYPE_DISCONNECT
					'Disconnection successful
					Exit
			End Select
			
			'Disconnect time reached?
			If MilliSecs() - begin >= disconnectTime Then Exit
		Wend
		
		'Reset the peer and destroy the host
		enet_peer_reset(_peer)
		enet_host_destroy(_host)
		_host = Null
		_peer = Null
	End Method

	Rem
	bbdoc: Aktualisiert den Client (empfngt und sendet Nachrichten)
	returns: Ein TNetClientEvent-Objekt, das beschreibt was passiert ist, oder Null wenn kein Ereignis stattfand
	about:
	waitTime:Int		(Optional) Zeit in ms die der Client auf Nachrichten wartet. (Standard = 0, das heisst wenn keine Nachrichten vorhanden sind, kehrt die Methode sofort zurck und wartet nicht weiter)
	End Rem
	Method Update:TNetClientEvent(waitTime:Int = 0)
		If _host = Null Then Return Null
		
		Local ev:ENetEvent = New ENetEvent
		If enet_host_service(_host, ev, waitTime) > 0
			Select ev.event
				Case ENET_EVENT_TYPE_RECEIVE
					DebugLog("Packet received")
					Return TNetClientEvent.ClientEvent(ev.event, TNetPacket.FromEnet(ev.packet), ev.channel)
					
				Case ENET_EVENT_TYPE_DISCONNECT
					DebugLog("Disconnected")
					Return TNetClientEvent.ClientEvent(ev.event, Null, 0)
			End Select
		Else
			Return Null
		End If
	End Method

	Rem
	bbdoc: Sendet ein Packet an den Server
	about:
	packet:TNetPacket		Das Packet/Die Daten die gesendet werden
	reliable:Int			(Optional) Bestimmt ob das Packet zwingend ankommen muss (Standard: Nein, gut fr Positions-bermittlung etc.)
	channel:Int				(Optional) Kanal ber den das Packet gesendet wird (Standard: 0, Wichtige reliable-Nachrichten sollten ber einen anderen Kanal gesendet werden als unwichtige Nachrichten, z.B. Kanal 1)
	compressed:Int			(Optional) Bestimmt ob die Daten vor der bermittlung komprimiert werden (Standard: Nein, lohnt sich nur bei sehr grossen Daten-Paketen, wie z.b. wenn man 100x100 Tiles in 1 Paket senden mchte).
	End Rem
	Method Send(packet:TNetPacket, reliable:Int = False, channel:Int = 0, compressed:Int = False)
		If packet = Null Then Return
		If _host = Null Or _peer = Null Then Return
		
		Local flags:Int = 0
		If reliable Then flags = ENET_PACKET_FLAG_RELIABLE
		
		Local p:Byte Ptr = packet.EnetPacket(flags, compressed)
		If p = Null Then Return
		
		enet_peer_send(_peer, channel, p)
	End Method

	Rem
	bbdoc: Sendet alle zwischengespeicherten Nachrichten so schnell wie mglich bzw. sofort
	about:
	Es ist besser Update zu verwenden, da Update ebenfalls alle Nachrichten sendet und auch Nachrichten empfngt
	End Rem
	Method Flush()
		If _host = Null Then Return
		enet_host_flush(_host)
	End Method
End Type