
Rem
bbdoc: Stellt ein Datenpaket dar, in das Werte geschrieben, oder von dem Werte gelesen werden knnen
End Rem
Type TNetPacket
	Field _packet:Byte Ptr
	Field _bank:TBank
	Field _stream:TBankStream

	Rem
	bbdoc: Erstellt ein neues Datenpaket, in welches Daten geschrieben werden knnen (zum Versand)
	End Rem
	Function Create:TNetPacket()
		Local packet:TNetPacket = New TNetPacket
		packet._bank = TBank.Create(0)
		packet._stream = TBankStream.Create(packet._bank)
		packet.WriteByte(0)
		Return packet
	End Function
	
	Rem
	bbdoc: Liest ein Datenpaket aus einer ENet-Struktur
	about:
	Die Funktion wird intern verwendet und muss nicht manuell aufgerufen werden
	End Rem
	Function FromEnet:TNetPacket(p:Byte Ptr)
		Local packet:TNetPacket = New TNetPacket
		packet._packet = p
		packet._bank = TBank.CreateStatic(enet_packet_data(p), enet_packet_size(p))
		packet._stream = TBankStream.Create(packet._bank)

		'Decompression
		Local cmp:Byte = packet.ReadByte()
		If cmp Then
			Local buf:Byte Ptr = MemAlloc(packet._bank.Size() - 1)
			Local l:Int = 0
			uncompress(buf, l, Varptr packet._bank.buf()[1], packet._bank.Size() - 1)
			
			packet._stream.Close()
			packet._bank = TBank.Create(l)
			packet._stream = TBankStream.Create(packet._bank)
			
			MemCopy(packet._bank.buf(), buf, l)
			MemFree(buf)
		End If
		
		Return packet
	End Function

	Method Delete()
		Destroy()
	End Method
	
	Rem
	bbdoc: Gibt den vom Paket verwendeten Speicher wieder frei
	about:
	Wird automatisch aufgerufen wenn ein Paket nicht mehr verwendet wird (GarbageCollector),
	kann aber auch manuell aufgerufen werden
	End Rem
	Method Destroy()
		If _stream <> Null Then
			_stream.Close()
			_stream = Null
		End If
		
		If _bank <> Null Then
			_bank = Null
		End If
	
		If _packet <> Null Then
			enet_packet_destroy(_packet)
			_packet = Null
			DebugLog("TNetPacket destroyed")
		End If
	End Method

	Rem
	bbdoc: Liest mehrere Bytes vom Paket in einen benutzerdefinierten Speicherbereich
	returns: Anzahl effektiv gelesene Bytes
	about:
	buf:Byte Ptr		Speicherbereich in dem die gelesenen Daten abgelegt werden
	count:Int			Anzahl Bytes die gelesen werden
	End Rem
	Method Read:Int(buf:Byte Ptr, count:Int)
		Return _stream.Read(buf, count)
	End Method
	
	Rem
	bbdoc: Liest ein Byte vom Paket
	End Rem
	Method ReadByte:Byte()
		Return _stream.ReadByte()
	End Method
	
	Rem
	bbdoc: Liest mehrere Bytes vom Paket in einen benutzerdefinierten Speicherbereich
	returns: Anzahl effektiv gelesene Bytes
	about:
	buf:Byte Ptr		Speicherbereich in dem die gelesenen Daten abgelegt werden
	count:Int			Anzahl Bytes die gelesen werden
	End Rem
	Method ReadBytes:Int(buf:Byte Ptr, count:Int)
		Return _stream.ReadBytes(buf, count)
	End Method

	Rem
	bbdoc: Liest ein Short vom Paket
	End Rem
	Method ReadShort:Short()
		Return _stream.ReadShort()
	End Method
	
	Rem
	bbdoc: Liest ein Int vom Paket
	End Rem
	Method ReadInt:Int()
		Return _stream.ReadInt()
	End Method

	Rem
	bbdoc: Liest ein Long vom Paket
	End Rem
	Method ReadLong:Long()
		Return _stream.ReadLong()
	End Method
	
	Rem
	bbdoc: Liest ein Float vom Paket
	End Rem
	Method ReadFloat:Float()
		Return _stream.ReadFloat()
	End Method
	
	Rem
	bbdoc: Liest ein Double vom Paket
	End Rem
	Method ReadDouble:Double()
		Return _stream.ReadDouble()
	End Method
	
	Rem
	bbdoc: Liest einen String vom Paket
	about:
	Liest ein Int mit der Lnge des String und anschliessend den String selber aus
	End Rem
	Method ReadString:String()
		Local l:Int = _stream.ReadInt()
		If l = -1 Then Return Null
		Return _stream.ReadString(l)
	End Method
	
	Rem
	bbdoc: Liest eine Zeile vom Paket
	End Rem
	Method ReadLine:String()
		Return _stream.ReadLine()
	End Method
	
	Rem
	bbdoc: Liest ein Objekt vom Paket (nicht getestet)
	End Rem
	Method ReadObject:Object()
		Return _stream.ReadObject()
	End Method

	Rem
	bbdoc: Schreibt mehrere Bytes in das Paket
	about:
	buf:Byte Ptr		Speicherbereich aus dem die Daten ins Paket gelesen werden
	count:Int			Anzahl Bytes die geschrieben/gelesen werden
	End Rem
	Method Write(buf:Byte Ptr, count:Int)
		_stream.Write(buf, count)
	End Method
	
	Rem
	bbdoc: Schreibt ein Byte in das Paket
	End Rem
	Method WriteByte(value:Byte)
		_stream.WriteByte(value)
	End Method
	
	Rem
	bbdoc: Schreibt mehrere Bytes in das Paket
	about:
	buf:Byte Ptr		Speicherbereich aus dem die Daten ins Paket gelesen werden
	count:Int			Anzahl Bytes die geschrieben/gelesen werden
	End Rem
	Method WriteBytes(buf:Byte Ptr, count:Int)
		_stream.WriteBytes(buf, count)
	End Method

	Rem
	bbdoc: Schreibt ein Short in das Paket
	End Rem
	Method WriteShort(value:Short)
		_stream.WriteShort(value)
	End Method
	
	Rem
	bbdoc: Schreibt ein Int in das Paket
	End Rem
	Method WriteInt(value:Int)
		_stream.WriteInt(value)
	End Method
	
	Rem
	bbdoc: Schreibt ein Long in das Paket
	End Rem
	Method WriteLong(value:Long)
		_stream.WriteLong(value)
	End Method
	
	Rem
	bbdoc: Schreibt ein Float in das Paket
	End Rem
	Method WriteFloat(value:Float)
		_stream.WriteFloat(value)
	End Method
	
	Rem
	bbdoc: Schreibt ein Double in das Paket
	End Rem
	Method WriteDouble(value:Double)
		_stream.WriteDouble(value)
	End Method
	
	Rem
	bbdoc: Schreibt ein String in das Paket
	about:
	Schreibt ein Int mit der Lnge des Strings und anschliessend den String selber
	End Rem
	Method WriteString(value:String)
		If value = 0
			_stream.WriteInt(-1)
			Return
		Else
			_stream.WriteInt(value.Length)
			_stream.WriteString(value)
		End If
	End Method
	
	Rem
	bbdoc: Schreibt eine Zeile in das Paket
	End Rem
	Method WriteLine(value:String)
		_stream.WriteLine(value)
	End Method
	
	Rem
	bbdoc: Schreibt ein Objekt in das Paket (nicht getestet)
	End Rem
	Method WriteObject(value:Object)
		_stream.WriteObject(value)
	End Method
	
	Rem
	bbdoc: Generiert eine ENet-Paket-Struktur
	about:
	Diese Methode wird intern verwendet und muss nicht manuell aufgerufen werden
	End Rem
	Method EnetPacket:Byte Ptr(enet_flags:Int = 0, compressed:Int = False)
		If _packet <> Null Then Return _packet
		If _bank = Null Then Return Null
		
		If compressed Then
			'Compression
			DebugStop
			Local cmp:Byte Ptr = MemAlloc(_bank.Size() - 1)
			Local cmpLen:Int = 0
			compress(cmp, cmpLen, Varptr _bank.buf()[1], _bank.Size() - 1)
			
			Local buf:Byte Ptr = MemAlloc(cmpLen + 1)
			MemCopy(Varptr buf[1], cmp, cmpLen)
			buf[0] = 1

			Local p:Byte Ptr = enet_packet_create(buf, cmpLen + 1, enet_flags)
			
			MemFree(cmp)
			MemFree(buf)
			
			Return p
		Else
			'No compression
			Return enet_packet_create(_bank.buf(), _bank.Size(), enet_flags)
		End If
	End Method

End Type