Rem
bbdoc: Klasse zum Betreiben eines Servers
End Rem
Type TNetServer
	Field _host:Byte Ptr
	Field _peers:TList
	
	Method New()
		_peers = New TList
	End Method

	Rem	
	bbdoc: Erstellt einen neuen Server welcher auf der angegebenen IP-Adresse und Port auf eingehende Verbindungen wartet
	returns: Entweder ein TNetServer-Objekt oder Null, falls der Server nicht gestartet werden konnte
	about:
	ip:String			IP-Adresse auf der der Server luft (Null, 127.0.0.1 oder localhost verwenden um jede lokale IP abzuhren)
	port:Int			Port auf der der Server luft (empfohlen: ber 1000 und mglichst hoch, z.B. 50000)
	maxConnections:Int	Anzahl maximaler Verbindungen/Spieler fr den Server
	End Rem
	Function Create:TNetServer(ip:String, port:Int, maxConnections:Int)
		Local addr:Byte Ptr = enet_address_create(ENET_HOST_ANY, port)
		If addr = Null Then Return Null
		
		If ip <> Null And ip <> "" And ip <> "localhost" And ip <> "127.0.0.1"
			enet_address_set_host(addr, ip)
		End If
		
		Local host:Byte Ptr = enet_host_create(addr, maxConnections, 0, 0)
		enet_address_destroy(addr)
		If host = Null Then Return Null
		
		Local sv:TNetServer = New TNetServer
		sv._host = host
		Return sv
	End Function

	Rem
	bbdoc: Beendet die Ausfhrung des Servers
	about:
	shutdownTime:Int		(Optional) Maximale Zeit in ms in der es erlaubt ist vor dem Beenden noch die letzten Nachrichten an Clients zu senden
	End Rem
	Method Shutdown(shutdownTime:Int = 5000)
		If _host = Null Then Return
	
		Local p:TNetPeer = Null
		
		For p = EachIn _peers
			enet_peer_disconnect(p.GetPeer())
		Next
		
		Local ev:ENetEvent = New ENetEvent
		
		Local begin:Int = MilliSecs()
		
		If shutdownTime > 0 Then
			While enet_host_service(_host, ev, shutdownTime) > 0
				Select ev.event
					Case ENET_EVENT_TYPE_CONNECT
						enet_peer_reset(ev.peer)
					
					Case ENET_EVENT_TYPE_RECEIVE
						enet_packet_destroy(ev.packet)
						
					Case ENET_EVENT_TYPE_DISCONNECT
						p = FindPeer(ev.peer)
						If p <> Null Then _peers.Remove(p)
						
				End Select
				
				If _peers.IsEmpty() Then Exit
				If MilliSecs() - begin >= shutdownTime Then Exit
			Wend
		EndIf

		If Not _peers.IsEmpty() Then
			For p = EachIn _peers
				enet_peer_reset(p)
			Next
			_peers.Clear()
		EndIf
		
		enet_host_destroy(_host)
		_host = Null
	End Method
	
	Rem
	bbdoc: Aktualisiert den Server (empfngt und sendet Nachrichten)
	returns: Ein TNetServerEvent-Objekt, das beschreibt was passiert ist, oder Null wenn kein Ereignis stattfand
	about:
	waitTime:Int		(Optional) Zeit in ms die der Server auf Nachrichten wartet. (Standard = 0, das heisst wenn keine Nachrichten vorhanden sind, kehrt die Methode sofort zurck und wartet nicht weiter)
	End Rem
	Method Update:TNetServerEvent(waitTime:Int = 0)
		If _host = Null Then Return Null
		
		Local p:TNetPeer = Null
		Local ev:ENetEvent = New ENetEvent
		
		If enet_host_service(_host, ev, waitTime) > 0
			Select ev.event
				'New client
				Case ENET_EVENT_TYPE_CONNECT
					p = TNetPeer.Create(ev.peer)
					_peers.AddLast(p)
					
					Return TNetServerEvent.ServerEvent(ev.event, Null, p, ev.channel)

					
				'Data from existing client				
				Case ENET_EVENT_TYPE_RECEIVE
					p = FindPeer(ev.peer)
					If p = Null Then Return Null
					
					Return TNetServerEvent.ServerEvent(ev.event, TNetPacket.FromEnet(ev.packet), p, ev.channel)

									
				'Client disconnected	
				Case ENET_EVENT_TYPE_DISCONNECT
					p = FindPeer(ev.peer)
					If p = Null Then Return Null
					_peers.Remove(p)
					
					Return TNetServerEvent.ServerEvent(ev.event, Null, p, ev.channel)
			End Select
		Else
			Return Null
		End If
	End Method

	
	'Finds a client/peer by enet peer struct
	Method FindPeer:TNetPeer(peer:Byte Ptr)
		For Local p:TNetPeer = EachIn _peers
			If p.GetPeer() = peer Then Return p
		Next
		Return Null
	End Method

		
	Rem
	bbdoc: Sendet ein Packet an einen bestimmten Client/Spieler
	about:
	peer:TNetPeer		Der Client/Spieler an den die Nachricht gesendet wird
	packet:TNetPacket	Das Packet/Die Daten die gesendet werden
	reliable:Int		(Optional) Bestimmt ob das Packet zwingend beim Spieler ankommen muss (Standard: Nein, gut fr Positions-bermittlung etc.)
	channel:Int			(Optional) Kanal ber den das Packet gesendet wird (Standard: 0, Wichtige reliable-Nachrichten sollten ber einen anderen Kanal gesendet werden als unwichtige Nachrichten, z.B. Kanal 1)
	compressed:Int		(Optional) Bestimmt ob die Daten vor der bermittlung komprimiert werden (Standard: Nein, lohnt sich nur bei sehr grossen Daten-Paketen, wie z.b. wenn man 100x100 Tiles in 1 Paket senden mchte).
	End Rem
	Method Send(peer:TNetPeer, packet:TNetPacket, reliable:Int = False, channel:Int = 0, compressed:Int = False)
		If _host = Null Or peer = Null Or packet = Null Then Return
		If peer.GetPeer() = Null Then Return
		
		Local flags:Int = 0
		If reliable Then flags = ENET_PACKET_FLAG_RELIABLE
		
		Local p:Byte Ptr = packet.EnetPacket(flags, compressed)
		If p = Null Then Return
		
		enet_peer_send(peer.GetPeer(), channel, p)
	End Method

	
	Rem	
	bbdoc: Sendet ein Packet an alle verbundenen Clients/Spieler
	about:
	packet:TNetPacket	Das Packet/Die Daten die gesendet werden
	reliable:Int		(Optional) Bestimmt ob das Packet zwingend beim Spieler ankommen muss (Standard: Nein, gut fr Positions-bermittlung etc.)
	channel:Int			(Optional) Kanal ber den das Packet gesendet wird (Standard: 0, Wichtige reliable-Nachrichten sollten ber einen anderen Kanal gesendet werden als unwichtige Nachrichten, z.B. Kanal 1)
	compressed:Int		(Optional) Bestimmt ob die Daten vor der bermittlung komprimiert werden (Standard: Nein, lohnt sich nur bei sehr grossen Daten-Paketen, wie z.b. wenn man 100x100 Tiles in 1 Paket senden mchte).
	End Rem
	Method Broadcast(packet:TNetPacket, reliable:Int = False, channel:Int = 0, compressed:Int = False)
		If _host = Null Or packet = Null Then Return
		
		Local flags:Int = 0
		If reliable Then flags = ENET_PACKET_FLAG_RELIABLE
		
		Local p:Byte Ptr = packet.EnetPacket(flags, compressed)
		If p = Null Then Return
		
		enet_host_broadcast(_host, channel, p)
	End Method

	Rem
	bbdoc: Gibt die Liste mit allen verbundenen Clients/Spielern zurck
	about:
	Die Liste sollte nicht manuell verndert werden, das heisst kein Lschen oder Hinzufgen eines Spielers. Dazu sollten die DisconnectClient/DropClient-Methoden benutzt werden.
	End Rem
	Method GetClients:TList()
		Return _peers
	End Method

	Rem
	bbdoc: Sendet alle zwischengespeicherten Nachrichten so schnell wie mglich bzw. sofort
	about:
	Es ist besser Update zu verwenden, da Update ebenfalls alle Nachrichten sendet und auch Nachrichten empfngt
	End Rem
	Method Flush()
		If _host = Null Then Return
		enet_host_flush(_host)
	End Method
	
	Rem
	bbdoc: Sendet eine Disconnect-Nachricht an den Client/Spieler und trennt anschliessend die Verbindung wenn die Nachricht besttigt wurde
	End Rem
	Method DisconnectClient(peer:TNetPeer)
		If peer = Null Or peer.GetPeer() = Null Then Return
		enet_peer_disconnect(peer.GetPeer())
	End Method
	
	Rem
	bbdoc: Trennt sofort die Verbindung mit einem Client. Der Client wird nicht benachrichtigt und merkt evtl. erst spter (Timeout) dass die Verbindung gekappt wurde.
	End Rem
	Method DropClient(peer:TNetPeer)
		If peer = Null Or peer.GetPeer() = Null Then Return
		enet_peer_reset(peer.GetPeer())
		_peers.Remove(peer)
	End Method
End Type